"""Import all built-in source connectors to trigger registration."""

from . import (
    email,  # stdlib imaplib — always register  # noqa: F401
    http,  # no extra needed — always register  # noqa: F401
)


def register_cloud_sources() -> None:
    """Try to import cloud connectors. Silently skip if extras not installed."""
    import contextlib

    with contextlib.suppress(ImportError):
        from . import s3  # noqa: F401
    with contextlib.suppress(ImportError):
        from . import azure  # noqa: F401
    with contextlib.suppress(ImportError):
        from . import gcs  # noqa: F401
